package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.params.DeviceInOutForm;
import com.qmth.sop.business.bean.params.DeviceInOutSubmitParam;
import com.qmth.sop.business.bean.result.DeviceInOutResult;
import com.qmth.sop.business.entity.SysDevice;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBDeviceInOut;
import com.qmth.sop.business.mapper.TBDeviceInOutMapper;
import com.qmth.sop.business.service.SequenceService;
import com.qmth.sop.business.service.SysDeviceService;
import com.qmth.sop.business.service.SysUserService;
import com.qmth.sop.business.service.TBDeviceInOutService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 设备出入库登记 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBDeviceInOutServiceImpl extends ServiceImpl<TBDeviceInOutMapper, TBDeviceInOut> implements TBDeviceInOutService {
    @Resource
    private SysDeviceService sysDeviceService;
    @Resource
    private SequenceService sequenceService;
    @Resource
    private SysUserService sysUserService;

    @Override
    public IPage<DeviceInOutResult> findDeviceInOutPageBySop(Long serviceUnitId, DeviceUsageTypeEnum usageType, Long userId, DeviceStatusEnum deviceStatus, Long inOutTimeStart, Long inOutTimeEnd, String deviceNo, String customName, String location, String address, String serialNo, Integer pageNumber, Integer pageSize) {
        deviceNo = SystemConstant.translateSpecificSign(deviceNo);
        customName = SystemConstant.translateSpecificSign(customName);
        location = SystemConstant.translateSpecificSign(location);
        address = SystemConstant.translateSpecificSign(address);
        serialNo = SystemConstant.translateSpecificSign(serialNo);

        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(requestUser.getId());

        return this.baseMapper.findDeviceInOutPageBySop(new Page<>(pageNumber, pageSize), serviceUnitId, usageType,
                userId, deviceStatus, inOutTimeStart, inOutTimeEnd, deviceNo, customName, location, address, serialNo, dpr);
    }

    @Override
    public List<DeviceInOutResult> findDeviceCanOutInfo(Long supplierId) {
        return this.baseMapper.findDeviceCanOutInfo(supplierId, DeviceStatusEnum.NORMAL, InOutTypeEnum.IN);
    }

    @Override
    public List<DeviceInOutResult> findDeviceCanInInfo(Long supplierId, String crmNo) {
        return this.baseMapper.findDeviceCanInInfo(supplierId, InOutTypeEnum.OUT, crmNo);
    }


    @Transactional
    @Override
    public void deviceInOutSubmit(DeviceInOutSubmitParam deviceInOutSubmitParam) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();

        Long inOutTime = deviceInOutSubmitParam.getInOutTime();
        InOutTypeEnum inOutType = deviceInOutSubmitParam.getInOutType();

        Long serviceUnitId = deviceInOutSubmitParam.getServiceUnitId();
        String crmNo = deviceInOutSubmitParam.getCrmNo();
        String sopNo = deviceInOutSubmitParam.getSopNo();

        DeviceUsageTypeEnum usageType;
        if (sopNo != null && sopNo.length() > 0) {
            usageType = DeviceUsageTypeEnum.PROJECT;
        } else {
            usageType = DeviceUsageTypeEnum.OTHER;
        }
        if (deviceInOutSubmitParam.getDeviceInOutFormList().size() > deviceInOutSubmitParam.getDeviceInOutFormList().stream().map(DeviceInOutForm::getDeviceNo).distinct().count()) {
            throw ExceptionResultEnum.ERROR.exception("出入库登记表中设备编号有重复");
        }

        List<DeviceInOutForm> deviceInOutFormList = deviceInOutSubmitParam.getDeviceInOutFormList();
        List<TBDeviceInOut> tbDeviceInOutList = deviceInOutFormList.stream().flatMap(e -> {
            TBDeviceInOut tbDeviceInOut = new TBDeviceInOut();
            if (serviceUnitId != null && serviceUnitId > 0) {
                tbDeviceInOut.setServiceId(serviceUnitId);
            }
            if (crmNo != null && crmNo.length() > 0) {
                tbDeviceInOut.setCrmNo(crmNo);
            }
            if (sopNo != null && sopNo.length() > 0) {
                tbDeviceInOut.setSopNo(sopNo);
            }
            try {
                tbDeviceInOut.setSerialNo(sequenceService.createSerialNo(SequenceEnum.DEVICE_IN_OUT_CODE));
            } catch (InterruptedException ex) {
                throw ExceptionResultEnum.ERROR.exception("流水号生成失败");
            }
            tbDeviceInOut.setUserId(requestUser.getId());
            tbDeviceInOut.setType(inOutType);
            tbDeviceInOut.setDeviceNo(e.getDeviceNo());
            tbDeviceInOut.setSupplierName(e.getSupplierName());
            tbDeviceInOut.setLocation(e.getLocation());
            tbDeviceInOut.setAddress(e.getAddress());
            tbDeviceInOut.setStatus(e.getDeviceStatus());
            tbDeviceInOut.setUsageType(usageType);
            tbDeviceInOut.setBasePhotoPath(e.getBasePhotoPath());
            tbDeviceInOut.setInOutTime(inOutTime);

            UpdateWrapper<SysDevice> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda()
                    .set(SysDevice::getLocation, e.getAddress())
                    .set(SysDevice::getScanCount, e.getScanCount())
                    .set(SysDevice::getStatus, e.getDeviceStatus())
                    .set(SysDevice::getBound, inOutType)
                    .eq(SysDevice::getSerialNo, e.getDeviceNo());
            sysDeviceService.update(updateWrapper);

            return Stream.of(tbDeviceInOut);
        }).collect(Collectors.toList());
        this.saveBatch(tbDeviceInOutList);
    }

    @Override
    public IPage<DeviceInOutResult> findDeviceInOutPageBySource(Long serviceUnitId, DeviceUsageTypeEnum usageType, Long userId, DeviceStatusEnum deviceStatus, Long inOutTimeStart, Long inOutTimeEnd, String deviceNo, String customName, String location, String address, String serialNo, Integer pageNumber, Integer pageSize) {
        deviceNo = SystemConstant.translateSpecificSign(deviceNo);
        customName = SystemConstant.translateSpecificSign(customName);
        location = SystemConstant.translateSpecificSign(location);
        address = SystemConstant.translateSpecificSign(address);
        serialNo = SystemConstant.translateSpecificSign(serialNo);

        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(requestUser.getId());

        return this.baseMapper.findDeviceInOutPageBySource(new Page<>(pageNumber, pageSize), serviceUnitId, usageType,
                userId, deviceStatus, inOutTimeStart, inOutTimeEnd, deviceNo, customName, location, address, serialNo, dpr);
    }
}